@@ -37,17 +37,13 @@ class DjangoWsgiFix(object):
             signals.request_finished.connect(close_connection)
 
 
-class WebtestExceptionHandler(WSGIHandler):
-    """ Show original traceback instead of HTML 500 page """
-    def handle_uncaught_exception(self, request, resolver, exc_info):
-        return HttpResponseServerError(self._get_traceback(exc_info))
-
-
 class DjangoTestApp(TestApp):
 
     def __init__(self, extra_environ=None, relative_to=None):
-        app = DjangoWsgiFix(AdminMediaHandler(WebtestExceptionHandler()))
-        super(DjangoTestApp, self).__init__(app, extra_environ, relative_to)
+        super(DjangoTestApp, self).__init__(self.get_wsgi_handler(), extra_environ, relative_to)
+
+    def get_wsgi_handler(self):
+        return DjangoWsgiFix(AdminMediaHandler(WSGIHandler()))
 
     def _update_environ(self, environ, user):
         if user:
@@ -70,23 +66,36 @@ class DjangoTestApp(TestApp):
         # Add any rendered template detail to the response.
         # If there was only one template rendered (the most likely case),
         # flatten the list to a single element.
-        for detail in ('template', 'context'):
-            if data.get(detail):
-                if len(data[detail]) == 1:
-                    setattr(response, detail, data[detail][0]);
-                else:
-                    setattr(response, detail, data[detail])
-            else:
-                setattr(response, detail, None)
+        def flattend(detail):
+            if len(data[detail]) == 1:
+                return data[detail][0]
+            return data[detail]
+
+        response.context = None
+        response.template = None
+        response.templates = data.get('templates', None)
+
+        if data.get('context'):
+            response.context = flattend('context')
+
+        if data.get('template'):
+            response.template = flattend('template')
+        elif data.get('templates'):
+            response.template = flattend('templates')
 
         return response
 
     def get(self, url, params=None, headers=None, extra_environ=None,
-            status=None, expect_errors=False, user=None):
+            status=None, expect_errors=False, user=None, auto_follow=False):
         extra_environ = self._update_environ(extra_environ, user)
-        return super(DjangoTestApp, self).get(
+        response = super(DjangoTestApp, self).get(
                   url, params, headers, extra_environ, status, expect_errors)
 
+        is_redirect = lambda r: r.status_int >= 300 and r.status_int < 400
+        while auto_follow and is_redirect(response):
+            response = response.follow()
+
+        return response
 
     def post(self, url, params='', headers=None, extra_environ=None,
              status=None, upload_files=None, expect_errors=False,
@@ -108,11 +117,13 @@ class WebTest(TestCase):
             and (optional) to disable CSRF checks
         '''
 
+        self._DEBUG_PROPAGATE_EXCEPTIONS = settings.DEBUG_PROPAGATE_EXCEPTIONS
         self._MIDDLEWARE_CLASSES = settings.MIDDLEWARE_CLASSES[:]
         self._AUTHENTICATION_BACKENDS = settings.AUTHENTICATION_BACKENDS[:]
 
         settings.MIDDLEWARE_CLASSES = list(settings.MIDDLEWARE_CLASSES)
         settings.AUTHENTICATION_BACKENDS = list(settings.AUTHENTICATION_BACKENDS)
+        settings.DEBUG_PROPAGATE_EXCEPTIONS = True
 
         if not self.csrf_checks:
             self._disable_csrf_checks()
@@ -124,6 +135,7 @@ class WebTest(TestCase):
         ''' Restore settings to before-patching state '''
         settings.MIDDLEWARE_CLASSES = self._MIDDLEWARE_CLASSES
         settings.AUTHENTICATION_BACKENDS = self._AUTHENTICATION_BACKENDS
+        settings.DEBUG_PROPAGATE_EXCEPTIONS = self._DEBUG_PROPAGATE_EXCEPTIONS
 
     def _setup_auth(self):
         ''' Setup REMOTE_USER authorization '''
